// *=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*
// ** Copyright UCAR (c) 1992 - 2020
// ** University Corporation for Atmospheric Research (UCAR)
// ** National Center for Atmospheric Research (NCAR)
// ** Research Applications Lab (RAL)
// ** P.O.Box 3000, Boulder, Colorado, 80307-3000, USA
// *=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*

////////////////////////////////////////////////////////////////////////

#ifndef  __THERMO_H__
#define  __THERMO_H__

////////////////////////////////////////////////////////////////////////

#include "vx_util.h"
#include "vx_log.h"

////////////////////////////////////////////////////////////////////////
/* molecular weight ratio M_v / M_d (water vapor / dry air) */
const double epsilon = 0.622;

////////////////////////////////////////////////////////////////////////

double virtual_temperature(double temperature, double mixing_ratio);

////////////////////////////////////////////////////////////////////////

double mixing_ratio(double partial_pressure, double pressure);

////////////////////////////////////////////////////////////////////////

double saturation_vapor_pressure(double temperature);

////////////////////////////////////////////////////////////////////////

double saturation_mixing_ratio(double pressure, double temperature);

////////////////////////////////////////////////////////////////////////

double relative_humidity_from_mixing_ratio(double mixing_ratio,
    double temperature, double pressure);

////////////////////////////////////////////////////////////////////////

double mixing_ratio_from_relative_humidity(double relative_humidity,
    double temperature, double pressure);

////////////////////////////////////////////////////////////////////////

void height_from_pressure(int nlev,
    double surface_pressure,
    double* virtual_temperature,
    double* pressure, double* height);

////////////////////////////////////////////////////////////////////////

#endif  // __THERMO_H__

////////////////////////////////////////////////////////////////////////
